#ifndef SHADER_H
#define SHADER_H

#include <errno.h>
#include <stdlib.h>
#ifdef _MSC_VER
#include <direct.h> //_getcwd
#define getcwd _getcwd
#else
#include <unistd.h> //getcwd
#endif
#include <stdio.h>
#include <string.h>
#include <GL/glew.h>

#define SHADER_MAXLEN (sizeof(char)*2048)

static GLchar *readfile(const GLchar * path)
{
    FILE *soubor;
    GLchar *text = malloc(SHADER_MAXLEN);
    if ((soubor = fopen(path, "rt")) == NULL) {
	char cCurrentPath[FILENAME_MAX];
	getcwd(cCurrentPath, sizeof(cCurrentPath));
	fprintf(stderr, "%s %i %s: ", __FILE__, __LINE__, cCurrentPath);
	perror(NULL);
	free(text);
	return NULL;
    }
    size_t nacteno;
    if ((nacteno =
	 fread(text, 1, SHADER_MAXLEN - 1, soubor)) == SHADER_MAXLEN - 1) {
	fprintf(stderr, "SHADER_MAXLEN rached\n");
    }
    if (!nacteno) {
	perror(NULL);
	fprintf(stderr, "No data readed\n");
	free(text);
	return NULL;
    }
    fclose(soubor);
    text[nacteno] = '\0';
    return text;
}

struct Shader;

typedef struct Shader {
    GLuint Program;
    void (*Use) (const struct Shader *shader);
    GLuint (*getAttribLocation) (const struct Shader *shader, const char * variableName);
    GLuint (*getUniformLocation) (const struct Shader *shader, const char * variableName);
} Shader;

static void Use(const Shader *shader)
{
    glUseProgram(shader->Program);
}

static GLuint getAttribLocation(const Shader *shader, const char * variableName)
{
    return glGetAttribLocation(shader->Program, variableName);
}

static GLuint getUniformLocation(const Shader *shader, const char * variableName)
{
    return glGetUniformLocation(shader->Program, variableName);
}

Shader shader(const GLchar * vertexPath, const GLchar * fragmentPath)
{
    GLuint vertex, fragment;
    GLint success;
    GLchar infoLog[512];
    const GLchar *shaderCodeArr[1];
    Shader shader;
    shader.Use = Use;
    shader.getAttribLocation = getAttribLocation;
    shader.getUniformLocation = getUniformLocation;
    shader.Program = 0;

    GLchar * const vShaderCode = readfile(vertexPath);
    GLchar * const fShaderCode = readfile(fragmentPath);
    if (!vShaderCode || !fShaderCode) {
	return shader;
    }
    /*  Vertex Shader */
    vertex = glCreateShader(GL_VERTEX_SHADER);
    shaderCodeArr[0] = vShaderCode;
    glShaderSource(vertex, 1, shaderCodeArr, NULL);
    glCompileShader(vertex);
    glGetShaderiv(vertex, GL_COMPILE_STATUS, &success);
    if (!success) {
	glGetShaderInfoLog(vertex, 512, NULL, infoLog);
	fprintf(stderr, "Chyba kompilace Vertext Shaderu\n%s\n",
		infoLog);
    }

    /* Fragment Shader */
    fragment = glCreateShader(GL_FRAGMENT_SHADER);
    shaderCodeArr[0] = fShaderCode;
    glShaderSource(fragment, 1, shaderCodeArr, NULL);
    glCompileShader(fragment);
    glGetShaderiv(fragment, GL_COMPILE_STATUS, &success);
    if (!success) {
	glGetShaderInfoLog(fragment, 512, NULL, infoLog);
	fprintf(stderr, "Chyba kompilace fragment shaderu\n%s\n",
		infoLog);
    }

    /* Shader Program */
    shader.Program = glCreateProgram();
    glAttachShader(shader.Program, vertex);
    glAttachShader(shader.Program, fragment);
    glLinkProgram(shader.Program);
    glGetProgramiv(shader.Program, GL_LINK_STATUS, &success);
    if (!success) {
	glGetProgramInfoLog(shader.Program, 512, NULL, infoLog);
	fprintf(stderr, "Chyba linkovani shader programu\n%s\n",
		infoLog);
	shader.Program = 0;
    }

    glDeleteShader(vertex);
    glDeleteShader(fragment);
    free(fShaderCode);
    free(vShaderCode);
    return shader;
}

#endif